# Source code of the Ubuntu-it website
# Copyright (C) 2018 Pietro Albini <pietroalbini@ubuntu.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; witout even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import uuid

import flask
import flask_openid


class SessionError(BaseException):
    pass


class Sessions:

    def __init__(self, db):
        self.db = db

    def create(self, nickname, teams):
        """Create a new session and return its ID"""
        id = str(uuid.uuid4())
        ip = flask.request.remote_addr

        self.db.update(
            "INSERT INTO auth_sessions (id, nickname, teams, ip) "
            "VALUES (?, ?, ?, ?)", id, nickname, ",".join(teams), ip,
        )
        return id

    def check(self, id):
        """Check if a session is valid and return its data"""
        data = self.db.query("SELECT nickname, teams, ip FROM auth_sessions WHERE id = ?", id)
        if not data:
            raise SessionError("La tua sessione è scaduta, accedi di nuovo.")

        nickname = data[0][0]
        teams = data[0][1].split(",")
        ip = data[0][2]

        if ip != flask.request.remote_addr:
            raise SessionError("Questa sessione è valida solo su un'altra rete, accedi di nuovo.")

        return {
            "nickname": data[0][0],
            "teams": data[0][1].split(","),
        }

    def delete(self, id):
        """Delete a session"""
        self.db.update("DELETE FROM auth_sessions WHERE id = ?;", id)


def prepare_blueprint(app):
    """Prepare the auth blueprint"""
    bp = flask.Blueprint("auth", __name__)

    oid = flask_openid.OpenID(app, safe_roots=[])
    sessions = Sessions(app.db)

    @app.before_request
    def check_auth():
        if "auth" in flask.session:
            try:
                data = sessions.check(flask.session["auth"])
            except SessionError as e:
                del flask.session["auth"]
                flask.flash(str(e), "error")
                return

            flask.g.auth_name = data["nickname"]
            flask.g.auth_teams = data["teams"]

    @oid.after_login
    def receive_openid(resp):
        flask.session["auth"] = sessions.create(resp.nickname, [])
        flask.flash("Benvenuto %s!" % resp.nickname, "success")
        return flask.redirect(flask.url_for("pages.index"))

    @bp.route("/login")
    @oid.loginhandler
    def login():
        if "auth_name" not in flask.g:
            return oid.try_login("https://login.ubuntu.com/+openid", ask_for=["nickname"])
        else:
            flask.flash("Hai già effettuato l'accesso!", "info")
            return flask.redirect(flask.url_for("pages.index"))

    @bp.route("/logout")
    def logout():
        sessions.delete(flask.session["auth"])
        del flask.session["auth"]

        flask.flash("La sessione è stata terminata correttamente.", "success")
        return flask.redirect(flask.url_for("pages.index"))

    return bp
